#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <math.h>

#include "../tui/tui.h"
#include "./plot_lib.h"

void zoom_centered(Settings* settings, double factor) {
  double x_center = (settings->x_min + settings->x_max) / 2.0; 
  double y_center = (settings->y_min + settings->y_max) / 2.0; 
  double width = settings->x_max - settings->x_min;
  double height = settings->y_max - settings->y_min;
  settings->x_min = x_center - factor * width / 2.0;
  settings->x_max = x_center + factor * width / 2.0;
  settings->y_min = y_center - factor * height / 2.0;
  settings->y_max = y_center + factor * height / 2.0;
}

void plot(Settings* settings, Function* functions, size_t num_functions) {
  tui_init();

  size_t time_step = 0;

  while (1) {
    if (stdin_has_changed()) {
      char c = read_from_stdin();
      if (c == 'q') {
        break;
      } else if (c == '+') {
        zoom_centered(settings, 1.0 / 1.2);
      } else if (c == '-') {
        zoom_centered(settings, 1.2);
      }
    }

    Size2 size = tui_size();

    tui_clear();

    double step_x = (settings->x_max - settings->x_min) / size.x;
    double step_y = (settings->y_max - settings->y_min) / size.y;

    size_t sleep_time_in_micro_seconds = 10000;
    double t = time_step * (sleep_time_in_micro_seconds / 1000000.0);

    for (size_t i = 0; i < num_functions; ++i) {
      Function* f = functions + i;
      for (size_t x = 0; x < size.x; ++x) {
        double x2 = settings->x_min + step_x * x;
        double y2 = f->f(x2, t);
        size_t y = (y2 - settings->y_min) / step_y;
        if (y < size.y) {
          Cell* cell = tui_cell_at(x, y);
          cell->content = '#';
          cell->text_color = f->fg_color;
          cell->background_color = f->bg_color;
        }
      }
    }

    tui_update();

    time_step++;

    usleep(sleep_time_in_micro_seconds);
  }

  tui_shutdown();
}
